;***********************************************************************
; 1750Hz tone generator for handhelds
; realized by baseline processor PIC 10F200 @ 4MHz
; assembled by MPASM (MPLAB-IDE 8.92)
;
;***********************************************************************
        list    p=pic10f200
        
        #include        "p10f200.inc"

;-----------------------------------------------------------------------
; some macros additionally to pseudo assembler instructions
; caution! multiple bytes not skipable
;----------------------------------------------------------------------- 
dfbnz	macro	source,target		;decrement and branch if not zero
	decfsz	source,f        	
	goto	target
	endm

bbs	macro	source,bit,target	;branch on bit set
	btfsc	source,bit              
	goto	target
	endm

bbns	macro	source,bit,target	;branch on bit not set
	btfss	source,bit              
	goto	target
	endm

Ident	macro	
	data	"1750Hz/20201202/DF1OG"
	endm

;-----------------------------------------------------------------------
; function & GPIO allocation:
; activation of tone output (GPIO_2) by pressing PTT switch
; twice consecutively to ground (GPIO_0) within 300msec
;-----------------------------------------------------------------------

;-----------------------------------------------------------------------
; configuration: ext reset off, code protection off, watchdog off
;-----------------------------------------------------------------------
        __CONFIG 	_MCLRE_OFF & _CP_OFF & _WDT_OFF
        
;-----------------------------------------------------------------------
; constants & variables
;-----------------------------------------------------------------------
half_1750	equ	.28    		;half periode for 1750Hz
delay_loop	equ	.160     	;delay loop ~100msec
max_time	equ	.3		;max time for depressed/released PTT
io_config	equ	b'00001011'	;GPIO_0, 1, 3 = input, GPIO_2 = output
option1		equ	b'00000000'	;wake up, pull-ups, prescaler TMR0
option2		equ	b'10000000'     ;no wake up, pull-ups, presaler TMR0

BlockStart
        cblock  0x10
	R0				;general purpose registers
	R1
	R2
	R3

        endc
BlockEnd
        
	if BlockEnd - BlockStart > 0x0F
	error "/aDatenbereich berschreitet 0x1F"
	endif

;***********************************************************************
; reset
;***********************************************************************      
RC_cal  org     0xff            ;processor reset vector, rolls over to 0x00

        res	1	        ;reserve rom location "movlw osccal" 

Reset   org     0x00            ;effective reset vector        

	movwf   OSCCAL          ;apply internal RC factory calibration
 
;----------------------------------------------------------------------
; initialization                
;----------------------------------------------------------------------
Init	bbs	STATUS,GPWUF,Main ;reset by change on GPIO pin?

	movlw	io_config	;GPIO configuration
 	tris	GPIO

	bcf	OSCCAL,0	;GPIO_2 applied to GPIO pin	

	b	Main90          ;arm wake-up and sleep
	
;-----------------------------------------------------------------------
; main program
;-----------------------------------------------------------------------
Main	movlw	option2	        ;disable wake-up
	option
	
; GPIO configuration again after wake-up from sleep = reset :-)
	movlw	io_config
 	tris	GPIO

        bcf	OSCCAL,0	;GPIO_2 applied to GPIO pin
        
 	bcf	STATUS,GPWUF	;reset status bit change on GPIO pin
	
;-------------------------------
; check on-time of depressed PTT
;------------------------------- 
Main10	call	GetPTT          ;Get PTT
        andlw	b'00000001'
	bnz	Main90          ;PTT really depressed?

;--------------------------------
	clrf	R3              ;reset time count
	
Main15	call	GetPTT          ;get PTT
	andlw	b'00000001'
	bnz	Main20          ;PTT released?

	movlw   -max_time       ;check time of depressed PTT
	addwf	R3,w
	bc	Main90		;time over	

	incf	R3,f		;increment time count
	
	b	Main15          ;next round
	
;--------------------------------
; check off-time of released PTT
;--------------------------------
Main20	clrf	R3              ;reset time count

Main25	call	GetPTT          ;get PTT
	andlw	b'00000001'
	bz	Main30          ;PTT depressed?

	movlw   -max_time       ;check time of released PTT
	addwf	R3,w
	bc	Main90		;time over	

	incf	R3,f		;increment time count
	
	b	Main25          ;next round

;-------------------------------
; PTT depressed again in order to start tone out
;-------------------------------
Main30	call	GetPTT          ;get PTT
	andlw	b'00000001'
	bnz	Main90          ;PTT depressed?
	  
;-------------------------------
; tone out
;-------------------------------
Main40	movlw	half_1750       ;load half periode for 1750Hz	
	movwf	R0
	
Main50	movfw	GPIO		;PTT released? - 1 cycle
	andlw	b'00000001'     ;mask out GPIO_0 - 1 cycle
	bnz	Main90		;PTT released again? - 3 cycles

	nop                     ;adjustment by additional 3 cycles 
	nop
	nop
		
	dfbnz	R0,Main50       ;decrement & loop - 3 cycles
 
 	movfw	GPIO
	xorlw	b'00000100'	;toggle tone output  			
	movwf	GPIO
	
	b	Main40          ;next half periode
	
;-------------------------------
; exit
;-------------------------------
Main90	bcf	GPIO,GP2	;avoid output current on GPIO_2 during sleep

	movfw	GPIO		;necessary GPIO dummy read
	
	movlw	option1		;enable wake-up again	
	option
	
Main99	sleep 			;wait for change on PTT
 	
;-----------------------------------------------------------------------
; get PTT and debounce
; returns 0 if PTT depressed
; returns 1 if PTT released
;-----------------------------------------------------------------------
GetPTT	movfw	GPIO		;get GPIO
        andlw	b'00000001'	;mask out GPIO_0
	movwf	R2		;save value

	call	Delay           ;debouncing

	movfw	GPIO		;get GPIO again
        andlw	b'00000001'	;mask out GPIO_0
	xorwf	R2,w		;PTT valid?
	bnz	GetPTT

	movfw	R2		;check PTT depressed/released?
	bnz	GetPTT1
	
GetPTT0	retlw	0               ;PTT released
	
GetPTT1	retlw	1	        ;PTT depressed
 
;-----------------------------------------------------------------------
; delay loop ~100 msec
;-----------------------------------------------------------------------
Delay	movlw	delay_loop 	;outer loop delay ~100msec
 	movwf	R1
 
Delay10	movlw	0		;load inner loop
	movwf	R0
 
Delay20	dfbnz	R0,Delay20	;inner loop - 3 cycles
 	
 	dfbnz	R1,Delay10      ;outer loop over? - 3 cycles

Delay99	retlw	0

;-----------------------------------------------------------------------
	Ident			;version info
;-----------------------------------------------------------------------
Page_End        
		if Page_End > 0xFE
		error "/aCode berschreitet 0xFE"
		endif

        end
    